package javax.swing.plaf.basic;

import java.awt.event.KeyEvent;
import javax.swing.JApplet;
import javax.swing.JDialog;
import javax.swing.JFrame;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusListener;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.ComponentEvent;
import java.util.Iterator;
import java.applet.Applet;
import javax.swing.JComboBox;
import sun.awt.UngrabEvent;
import java.awt.AWTEvent;
import javax.swing.event.ChangeEvent;
import sun.awt.SunToolkit;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.WindowListener;
import java.awt.event.ComponentListener;
import java.awt.event.AWTEventListener;
import javax.swing.event.ChangeListener;
import javax.swing.JMenuBar;
import javax.swing.JRootPane;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import javax.swing.JMenuItem;
import java.util.Arrays;
import javax.swing.JMenu;
import javax.swing.event.MenuKeyEvent;
import javax.swing.event.PopupMenuEvent;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.MenuElement;
import javax.swing.SwingUtilities;
import javax.swing.ActionMap;
import javax.swing.ComponentInputMap;
import javax.swing.InputMap;
import javax.swing.MenuSelectionManager;
import sun.awt.AppContext;
import javax.swing.plaf.UIResource;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.event.MenuKeyListener;
import javax.swing.event.PopupMenuListener;
import javax.swing.JPopupMenu;
import javax.swing.plaf.PopupMenuUI;

public class BasicPopupMenuUI extends PopupMenuUI
{
    static final StringBuilder MOUSE_GRABBER_KEY;
    static final StringBuilder MENU_KEYBOARD_HELPER_KEY;
    protected JPopupMenu popupMenu;
    private transient PopupMenuListener popupMenuListener;
    private MenuKeyListener menuKeyListener;
    private static boolean checkedUnpostPopup;
    private static boolean unpostPopup;
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicPopupMenuUI();
    }
    
    public BasicPopupMenuUI() {
        this.popupMenu = null;
        this.popupMenuListener = null;
        this.menuKeyListener = null;
        BasicLookAndFeel.needsEventHelper = true;
        LookAndFeel lookAndFeel = UIManager.getLookAndFeel();
        if ((lookAndFeel instanceof BasicLookAndFeel)) {
            ((BasicLookAndFeel)lookAndFeel).installAWTEventListener();
        }
    }
    
    @Override
    public void installUI(JComponent component) {
        this.popupMenu = (JPopupMenu)component;
        this.installDefaults();
        this.installListeners();
        this.installKeyboardActions();
    }
    
    public void installDefaults() {
        if (this.popupMenu.getLayout() == null || this.popupMenu.getLayout() instanceof UIResource) {
            this.popupMenu.setLayout(new DefaultMenuLayout(this.popupMenu, 1));
        }
        LookAndFeel.installProperty(this.popupMenu, "opaque", Boolean.TRUE);
        LookAndFeel.installBorder(this.popupMenu, "PopupMenu.border");
        LookAndFeel.installColorsAndFont(this.popupMenu, "PopupMenu.background", "PopupMenu.foreground", "PopupMenu.font");
    }
    
    protected void installListeners() {
        if (this.popupMenuListener == null) {
            this.popupMenuListener = new BasicPopupMenuUI.BasicPopupMenuListener();
        }
        this.popupMenu.addPopupMenuListener(this.popupMenuListener);
        if (this.menuKeyListener == null) {
            this.menuKeyListener = new BasicPopupMenuUI.BasicMenuKeyListener();
        }
        this.popupMenu.addMenuKeyListener(this.menuKeyListener);
        AppContext appContext = AppContext.getAppContext();
        synchronized (BasicPopupMenuUI.MOUSE_GRABBER_KEY) {
            if ((BasicPopupMenuUI.MouseGrabber)appContext.get(BasicPopupMenuUI.MOUSE_GRABBER_KEY) == null) {
                appContext.put(BasicPopupMenuUI.MOUSE_GRABBER_KEY, new BasicPopupMenuUI.MouseGrabber());
            }
        }
        synchronized (BasicPopupMenuUI.MENU_KEYBOARD_HELPER_KEY) {
            if ((BasicPopupMenuUI.MenuKeyboardHelper)appContext.get(BasicPopupMenuUI.MENU_KEYBOARD_HELPER_KEY) == null) {
                BasicPopupMenuUI.MenuKeyboardHelper menuKeyboardHelper = new BasicPopupMenuUI.MenuKeyboardHelper();
                appContext.put(BasicPopupMenuUI.MENU_KEYBOARD_HELPER_KEY, menuKeyboardHelper);
                MenuSelectionManager.defaultManager().addChangeListener(menuKeyboardHelper);
            }
        }
    }
    
    protected void installKeyboardActions() {
    }
    
    static InputMap getInputMap(JPopupMenu popupMenu, JComponent component) {
        ComponentInputMap componentInputMap = null;
        Object[] keys = (Object[])(Object[])UIManager.get("PopupMenu.selectedWindowInputMapBindings");
        if (keys != null) {
            componentInputMap = LookAndFeel.makeComponentInputMap(component, keys);
            if (!popupMenu.getComponentOrientation().isLeftToRight()) {
                Object[] keys2 = (Object[])(Object[])UIManager.get("PopupMenu.selectedWindowInputMapBindings.RightToLeft");
                if (keys2 != null) {
                    ComponentInputMap componentInputMap2 = LookAndFeel.makeComponentInputMap(component, keys2);
                    componentInputMap2.setParent(componentInputMap);
                    componentInputMap = componentInputMap2;
                }
            }
        }
        return componentInputMap;
    }
    
    static ActionMap getActionMap() {
        return LazyActionMap.getActionMap(BasicPopupMenuUI.class, "PopupMenu.actionMap");
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicPopupMenuUI.Actions("cancel"));
        lazyActionMap.put(new BasicPopupMenuUI.Actions("selectNext"));
        lazyActionMap.put(new BasicPopupMenuUI.Actions("selectPrevious"));
        lazyActionMap.put(new BasicPopupMenuUI.Actions("selectParent"));
        lazyActionMap.put(new BasicPopupMenuUI.Actions("selectChild"));
        lazyActionMap.put(new BasicPopupMenuUI.Actions("return"));
        BasicLookAndFeel.installAudioActionMap(lazyActionMap);
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults();
        this.uninstallListeners();
        this.uninstallKeyboardActions();
        this.popupMenu = null;
    }
    
    protected void uninstallDefaults() {
        LookAndFeel.uninstallBorder(this.popupMenu);
    }
    
    protected void uninstallListeners() {
        if (this.popupMenuListener != null) {
            this.popupMenu.removePopupMenuListener(this.popupMenuListener);
        }
        if (this.menuKeyListener != null) {
            this.popupMenu.removeMenuKeyListener(this.menuKeyListener);
        }
    }
    
    protected void uninstallKeyboardActions() {
        SwingUtilities.replaceUIActionMap(this.popupMenu, null);
        SwingUtilities.replaceUIInputMap(this.popupMenu, 2, null);
    }
    
    static MenuElement getFirstPopup() {
        MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
        MenuElement menuElement = null;
        for (int n = 0; menuElement == null && n < selectedPath.length; ++n) {
            if ((selectedPath[n] instanceof JPopupMenu)) {
                menuElement = selectedPath[n];
            }
        }
        return menuElement;
    }
    
    static JPopupMenu getLastPopup() {
        MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
        JPopupMenu popupMenu = null;
        for (int n = selectedPath.length - 1; popupMenu == null && n >= 0; --n) {
            if ((selectedPath[n] instanceof JPopupMenu)) {
                popupMenu = (JPopupMenu)selectedPath[n];
            }
        }
        return popupMenu;
    }
    
    static List<JPopupMenu> getPopups() {
        MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
        ArrayList list = new ArrayList(selectedPath.length);
        for (MenuElement menuElement : selectedPath) {
            if ((menuElement instanceof JPopupMenu)) {
                list.add((JPopupMenu)menuElement);
            }
        }
        return list;
    }
    
    @Override
    public boolean isPopupTrigger(MouseEvent mouseEvent) {
        return mouseEvent.getID() == 502 && (mouseEvent.getModifiers() & 0x4) != 0x0;
    }
    
    private static boolean checkInvokerEqual(MenuElement menuElement, MenuElement menuElement2) {
        Component component = menuElement.getComponent();
        Component component2 = menuElement2.getComponent();
        if ((component instanceof JPopupMenu)) {
            component = ((JPopupMenu)component).getInvoker();
        }
        if ((component2 instanceof JPopupMenu)) {
            component2 = ((JPopupMenu)component2).getInvoker();
        }
        return component == component2;
    }
    
    private static MenuElement nextEnabledChild(MenuElement[] array, int i, int n) {
        Component component;
        for (; i <= n; ++i) {
            if (array[i] != null) {
                component = array[i].getComponent();
                if (component != null && (component.isEnabled() || UIManager.getBoolean("MenuItem.disabledAreNavigable")) && component.isVisible()) {
                    return array[i];
                }
            }
        }
        return null;
    }
    
    private static MenuElement previousEnabledChild(MenuElement[] array, int i, int n) {
        Component component;
        for (; i >= n; --i) {
            if (array[i] != null) {
                component = array[i].getComponent();
                if (component != null && (component.isEnabled() || UIManager.getBoolean("MenuItem.disabledAreNavigable")) && component.isVisible()) {
                    return array[i];
                }
            }
        }
        return null;
    }
    
    static MenuElement findEnabledChild(MenuElement[] array, int n, boolean b) {
        MenuElement menuElement;
        if (b) {
            menuElement = nextEnabledChild(array, n + 1, array.length - 1);
            if (menuElement == null) {
                menuElement = nextEnabledChild(array, 0, n - 1);
            }
        }
        else {
            menuElement = previousEnabledChild(array, n - 1, 0);
            if (menuElement == null) {
                menuElement = previousEnabledChild(array, array.length - 1, n + 1);
            }
        }
        return menuElement;
    }
    
    static MenuElement findEnabledChild(MenuElement[] array, MenuElement menuElement, boolean b) {
        int i = 0;
        while (i < array.length) {
            if (array[i] == menuElement) {
                return findEnabledChild(array, i, b);
            }
            ++i;
        }
        return null;
    }
    
    static boolean access$300(MenuElement menuElement, MenuElement menuElement2) {
        return checkInvokerEqual(menuElement, menuElement2);
    }
    
    static {
        MOUSE_GRABBER_KEY = new StringBuilder("javax.swing.plaf.basic.BasicPopupMenuUI.MouseGrabber");
        MENU_KEYBOARD_HELPER_KEY = new StringBuilder("javax.swing.plaf.basic.BasicPopupMenuUI.MenuKeyboardHelper");
    }
    
    private class BasicPopupMenuListener implements PopupMenuListener
    {
        @Override
        public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
        }
        
        @Override
        public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
        }
        
        @Override
        public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
            BasicLookAndFeel.playSound((JPopupMenu)popupMenuEvent.getSource(), "PopupMenu.popupSound");
        }
    }
    
    private class BasicMenuKeyListener implements MenuKeyListener
    {
        MenuElement menuToOpen;
        
        private BasicMenuKeyListener() {
            this.menuToOpen = null;
        }
        
        @Override
        public void menuKeyTyped(MenuKeyEvent menuKeyEvent) {
            if (this.menuToOpen != null) {
                JPopupMenu popupMenu = ((JMenu)this.menuToOpen).getPopupMenu();
                MenuElement enabledChild = BasicPopupMenuUI.findEnabledChild(popupMenu.getSubElements(), -1, true);
                ArrayList list = new ArrayList(Arrays.asList(menuKeyEvent.getPath()));
                list.add(this.menuToOpen);
                list.add(popupMenu);
                if (enabledChild != null) {
                    list.add(enabledChild);
                }
                MenuSelectionManager.defaultManager().setSelectedPath((MenuElement[])list.toArray(new MenuElement[0]));
                menuKeyEvent.consume();
            }
            this.menuToOpen = null;
        }
        
        @Override
        public void menuKeyPressed(MenuKeyEvent menuKeyEvent) {
            int keyChar = menuKeyEvent.getKeyChar();
            if (!Character.isLetterOrDigit((char)keyChar)) {
                return;
            }
            MenuSelectionManager menuSelectionManager = menuKeyEvent.getMenuSelectionManager();
            MenuElement[] path = menuKeyEvent.getPath();
            MenuElement[] subElements = BasicPopupMenuUI.this.popupMenu.getSubElements();
            int n = -1;
            int n2 = 0;
            int n3 = -1;
            int[] array = null;
            for (int i = 0; i < subElements.length; ++i) {
                if (subElements[i] instanceof JMenuItem) {
                    JMenuItem menuItem = (JMenuItem)subElements[i];
                    int mnemonic = menuItem.getMnemonic();
                    if (menuItem.isEnabled() && menuItem.isVisible() && this.lower((char)keyChar) == this.lower(mnemonic)) {
                        if (n2 == 0) {
                            n3 = i;
                            ++n2;
                        }
                        else {
                            if (array == null) {
                                array = new int[subElements.length];
                                array[0] = n3;
                            }
                            array[n2++] = i;
                        }
                    }
                    if (menuItem.isArmed() || menuItem.isSelected()) {
                        n = n2 - 1;
                    }
                }
            }
            if (n2 != 0) {
                if (n2 == 1) {
                    JMenuItem menuToOpen = (JMenuItem)subElements[n3];
                    if ((menuToOpen instanceof JMenu)) {
                        this.menuToOpen = menuToOpen;
                    }
                    else if (menuToOpen.isEnabled()) {
                        menuSelectionManager.clearSelectedPath();
                        menuToOpen.doClick();
                    }
                    menuKeyEvent.consume();
                }
                else {
                    MenuElement menuElement = subElements[array[(n + 1) % n2]];
                    MenuElement[] selectedPath = new MenuElement[path.length + 1];
                    System.arraycopy(path, 0, selectedPath, 0, path.length);
                    selectedPath[path.length] = menuElement;
                    menuSelectionManager.setSelectedPath(selectedPath);
                    menuKeyEvent.consume();
                }
            }
            return;
        }
        
        @Override
        public void menuKeyReleased(MenuKeyEvent menuKeyEvent) {
        }
        
        private char lower(char ch) {
            return Character.toLowerCase(ch);
        }
        
        private char lower(int n) {
            return Character.toLowerCase((char)n);
        }
    }
    
    private static class Actions extends UIAction
    {
        private static final String CANCEL = "cancel";
        private static final String SELECT_NEXT = "selectNext";
        private static final String SELECT_PREVIOUS = "selectPrevious";
        private static final String SELECT_PARENT = "selectParent";
        private static final String SELECT_CHILD = "selectChild";
        private static final String RETURN = "return";
        private static final boolean FORWARD = true;
        private static final boolean BACKWARD = false;
        private static final boolean PARENT = false;
        private static final boolean CHILD = true;
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            String name = this.getName();
            if (name == "cancel") {
                this.cancel();
            }
            else if (name == "selectNext") {
                this.selectItem(true);
            }
            else if (name == "selectPrevious") {
                this.selectItem(false);
            }
            else if (name == "selectParent") {
                this.selectParentChild(false);
            }
            else if (name == "selectChild") {
                this.selectParentChild(true);
            }
            else if (name == "return") {
                this.doReturn();
            }
        }
        
        private void doReturn() {
            Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
            if (focusOwner != null && !(focusOwner instanceof JRootPane)) {
                return;
            }
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            MenuElement[] selectedPath = defaultManager.getSelectedPath();
            if (selectedPath.length > 0) {
                MenuElement menuElement = selectedPath[selectedPath.length - 1];
                if ((menuElement instanceof JMenu)) {
                    MenuElement[] selectedPath2 = new MenuElement[selectedPath.length + 1];
                    System.arraycopy(selectedPath, 0, selectedPath2, 0, selectedPath.length);
                    selectedPath2[selectedPath.length] = ((JMenu)menuElement).getPopupMenu();
                    defaultManager.setSelectedPath(selectedPath2);
                }
                else if ((menuElement instanceof JMenuItem)) {
                    JMenuItem menuItem = (JMenuItem)menuElement;
                    if ((menuItem.getUI() instanceof BasicMenuItemUI)) {
                        ((BasicMenuItemUI)menuItem.getUI()).doClick(defaultManager);
                    }
                    else {
                        defaultManager.clearSelectedPath();
                        menuItem.doClick(0);
                    }
                }
            }
            return;
        }
        
        private void selectParentChild(boolean b) {
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            MenuElement[] selectedPath = defaultManager.getSelectedPath();
            int length = selectedPath.length;
            if (!b) {
                int n = length - 1;
                if (length > 2 && (selectedPath[n] instanceof JPopupMenu || selectedPath[--n] instanceof JPopupMenu) && !((JMenu)selectedPath[n - 1]).isTopLevelMenu()) {
                    MenuElement[] selectedPath3 = new MenuElement[n];
                    System.arraycopy(selectedPath, 0, selectedPath3, 0, n);
                    defaultManager.setSelectedPath(selectedPath3);
                    return;
                }
            }
            else if (length > 0 && selectedPath[length - 1] instanceof JMenu && !((JMenu)selectedPath[length - 1]).isTopLevelMenu()) {
                JPopupMenu popupMenu = ((JMenu)selectedPath[length - 1]).getPopupMenu();
                MenuElement enabledChild = BasicPopupMenuUI.findEnabledChild(popupMenu.getSubElements(), -1, true);
                MenuElement[] selectedPath2;
                if (enabledChild == null) {
                    selectedPath2 = new MenuElement[length + 1];
                }
                else {
                    selectedPath2 = new MenuElement[length + 2];
                    selectedPath2[length + 1] = enabledChild;
                }
                System.arraycopy(selectedPath, 0, selectedPath2, 0, length);
                selectedPath2[length] = popupMenu;
                defaultManager.setSelectedPath(selectedPath2);
                return;
            }
            if (length > 1 && selectedPath[0] instanceof JMenuBar) {
                MenuElement menuElement = selectedPath[1];
                MenuElement enabledChild2 = BasicPopupMenuUI.findEnabledChild(selectedPath[0].getSubElements(), menuElement, b);
                if (enabledChild2 != null && enabledChild2 != menuElement) {
                    MenuElement[] selectedPath4;
                    if (length == 2) {
                        selectedPath4 = new MenuElement[] { selectedPath[0], enabledChild2 };
                    }
                    else {
                        selectedPath4 = new MenuElement[] { selectedPath[0], enabledChild2, ((JMenu)enabledChild2).getPopupMenu() };
                    }
                    defaultManager.setSelectedPath(selectedPath4);
                }
            }
        }
        
        private void selectItem(boolean b) {
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            MenuElement[] selectedPath = defaultManager.getSelectedPath();
            if (selectedPath.length == 0) {
                return;
            }
            int length = selectedPath.length;
            if (length == 1 && selectedPath[0] instanceof JPopupMenu) {
                JPopupMenu popupMenu2 = (JPopupMenu)selectedPath[0];
                defaultManager.setSelectedPath(new MenuElement[] { popupMenu2, BasicPopupMenuUI.findEnabledChild(popupMenu2.getSubElements(), -1, b) });
            }
            else if (length == 2 && selectedPath[0] instanceof JMenuBar && selectedPath[1] instanceof JMenu) {
                JPopupMenu popupMenu = ((JMenu)selectedPath[1]).getPopupMenu();
                MenuElement enabledChild3 = BasicPopupMenuUI.findEnabledChild(popupMenu.getSubElements(), -1, true);
                MenuElement[] selectedPath4;
                if (enabledChild3 != null) {
                    selectedPath4 = new MenuElement[] { null, null, null, enabledChild3 };
                }
                else {
                    selectedPath4 = new MenuElement[3];
                }
                System.arraycopy(selectedPath, 0, selectedPath4, 0, 2);
                selectedPath4[2] = popupMenu;
                defaultManager.setSelectedPath(selectedPath4);
            }
            else if (selectedPath[length - 1] instanceof JPopupMenu && selectedPath[length - 2] instanceof JMenu) {
                JMenu menu = (JMenu)selectedPath[length - 2];
                MenuElement enabledChild = BasicPopupMenuUI.findEnabledChild(menu.getPopupMenu().getSubElements(), -1, b);
                if (enabledChild != null) {
                    MenuElement[] selectedPath3 = new MenuElement[length + 1];
                    System.arraycopy(selectedPath, 0, selectedPath3, 0, length);
                    selectedPath3[length] = enabledChild;
                    defaultManager.setSelectedPath(selectedPath3);
                }
                else if (length > 2 && selectedPath[length - 3] instanceof JPopupMenu) {
                    MenuElement enabledChild2 = BasicPopupMenuUI.findEnabledChild(((JPopupMenu)selectedPath[length - 3]).getSubElements(), menu, b);
                    if (enabledChild2 != null && enabledChild2 != menu) {
                        MenuElement[] selectedPath2 = new MenuElement[length - 1];
                        System.arraycopy(selectedPath, 0, selectedPath2, 0, length - 2);
                        selectedPath2[length - 2] = enabledChild2;
                        defaultManager.setSelectedPath(selectedPath2);
                    }
                }
            }
            else {
                MenuElement[] subElements = selectedPath[length - 2].getSubElements();
                MenuElement menuElement = BasicPopupMenuUI.findEnabledChild(subElements, selectedPath[length - 1], b);
                if (menuElement == null) {
                    menuElement = BasicPopupMenuUI.findEnabledChild(subElements, -1, b);
                }
                if (menuElement != null) {
                    selectedPath[length - 1] = menuElement;
                    defaultManager.setSelectedPath(selectedPath);
                }
            }
            return;
        }
        
        private void cancel() {
            JPopupMenu lastPopup = BasicPopupMenuUI.getLastPopup();
            if (lastPopup != null) {
                lastPopup.putClientProperty("JPopupMenu.firePopupMenuCanceled", Boolean.TRUE);
            }
            if ("hideMenuTree".equals(UIManager.getString("Menu.cancelMode"))) {
                MenuSelectionManager.defaultManager().clearSelectedPath();
            }
            else {
                this.shortenSelectedPath();
            }
        }
        
        private void shortenSelectedPath() {
            MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
            if (selectedPath.length <= 2) {
                MenuSelectionManager.defaultManager().clearSelectedPath();
                return;
            }
            int length = 2;
            MenuElement menuElement = selectedPath[selectedPath.length - 1];
            JPopupMenu lastPopup = BasicPopupMenuUI.getLastPopup();
            if (menuElement == lastPopup) {
                MenuElement menuElement2 = selectedPath[selectedPath.length - 2];
                if ((menuElement2 instanceof JMenu)) {
                    if (((JMenu)menuElement2).isEnabled() && lastPopup.getComponentCount() > 0) {
                        length = 1;
                    }
                    else {
                        length = 3;
                    }
                }
            }
            if (selectedPath.length - length <= 2 && !UIManager.getBoolean("Menu.preserveTopLevelSelection")) {
                length = selectedPath.length;
            }
            MenuElement[] selectedPath2 = new MenuElement[selectedPath.length - length];
            System.arraycopy(selectedPath, 0, selectedPath2, 0, selectedPath.length - length);
            MenuSelectionManager.defaultManager().setSelectedPath(selectedPath2);
            return;
        }
    }
    
    static class MouseGrabber implements ChangeListener, AWTEventListener, ComponentListener, WindowListener
    {
        Window grabbedWindow;
        MenuElement[] lastPathSelected;
        
        public MouseGrabber() {
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            defaultManager.addChangeListener(this);
            this.lastPathSelected = defaultManager.getSelectedPath();
            if (this.lastPathSelected.length != 0) {
                this.grabWindow(this.lastPathSelected);
            }
        }
        
        void uninstall() {
            synchronized (BasicPopupMenuUI.MOUSE_GRABBER_KEY) {
                MenuSelectionManager.defaultManager().removeChangeListener(this);
                this.ungrabWindow();
                AppContext.getAppContext().remove(BasicPopupMenuUI.MOUSE_GRABBER_KEY);
            }
        }
        
        void grabWindow(MenuElement[] array) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    defaultToolkit.addAWTEventListener(BasicPopupMenuUI.MouseGrabber.this, -2147352464L);
                    return null;
                }
            });
            Component c = array[0].getComponent();
            if ((c instanceof JPopupMenu)) {
                c = ((JPopupMenu)c).getInvoker();
            }
            this.grabbedWindow = ((c instanceof Window) ? ((Window)c) : SwingUtilities.getWindowAncestor(c));
            if (this.grabbedWindow != null) {
                if ((defaultToolkit instanceof SunToolkit)) {
                    ((SunToolkit)defaultToolkit).grab(this.grabbedWindow);
                }
                else {
                    this.grabbedWindow.addComponentListener(this);
                    this.grabbedWindow.addWindowListener(this);
                }
            }
        }
        
        void ungrabWindow() {
            AccessController.doPrivileged(new PrivilegedAction<Object>() {
                
                @Override
                public Object run() {
                    tk.removeAWTEventListener(BasicPopupMenuUI.MouseGrabber.this);
                    return null;
                }
            });
            this.realUngrabWindow();
        }
        
        void realUngrabWindow() {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            if (this.grabbedWindow != null) {
                if ((defaultToolkit instanceof SunToolkit)) {
                    ((SunToolkit)defaultToolkit).ungrab(this.grabbedWindow);
                }
                else {
                    this.grabbedWindow.removeComponentListener(this);
                    this.grabbedWindow.removeWindowListener(this);
                }
                this.grabbedWindow = null;
            }
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
            if (this.lastPathSelected.length == 0 && selectedPath.length != 0) {
                this.grabWindow(selectedPath);
            }
            if (this.lastPathSelected.length != 0 && selectedPath.length == 0) {
                this.ungrabWindow();
            }
            this.lastPathSelected = selectedPath;
        }
        
        @Override
        public void eventDispatched(AWTEvent awtEvent) {
            if ((awtEvent instanceof UngrabEvent)) {
                this.cancelPopupMenu();
                return;
            }
            if (!(awtEvent instanceof MouseEvent)) {
                return;
            }
            MouseEvent mouseEvent = (MouseEvent)awtEvent;
            Component component = mouseEvent.getComponent();
            switch (mouseEvent.getID()) {
                case 501: {
                    if (this.isInPopup(component) || (component instanceof JMenu && ((JMenu)component).isSelected())) {
                        return;
                    }
                    if (!(component instanceof JComponent) || ((JComponent)component).getClientProperty("doNotCancelPopup") != BasicComboBoxUI.HIDE_POPUP_KEY) {
                        this.cancelPopupMenu();
                        if (UIManager.getBoolean("PopupMenu.consumeEventOnClose") && !(component instanceof MenuElement)) {
                            mouseEvent.consume();
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 502: {
                    if (!(component instanceof MenuElement) && this.isInPopup(component)) {
                        break;
                    }
                    if (component instanceof JMenu || !(component instanceof JMenuItem)) {
                        MenuSelectionManager.defaultManager().processMouseEvent(mouseEvent);
                        break;
                    }
                    break;
                }
                case 506: {
                    if (!(component instanceof MenuElement) && this.isInPopup(component)) {
                        break;
                    }
                    MenuSelectionManager.defaultManager().processMouseEvent(mouseEvent);
                    break;
                }
                case 507: {
                    if (this.isInPopup(component) || (component instanceof JComboBox && ((JComboBox)component).isPopupVisible())) {
                        return;
                    }
                    this.cancelPopupMenu();
                    break;
                }
            }
            return;
        }
        
        boolean isInPopup(Component parent) {
            while (parent != null && !(parent instanceof Applet) && !(parent instanceof Window)) {
                if ((parent instanceof JPopupMenu)) {
                    return true;
                }
                parent = parent.getParent();
            }
            return false;
        }
        
        void cancelPopupMenu() {
            try {
                Iterator<JPopupMenu> iterator = BasicPopupMenuUI.getPopups().iterator();
                while (iterator.hasNext()) {
                    ((JPopupMenu)iterator.next()).putClientProperty("JPopupMenu.firePopupMenuCanceled", Boolean.TRUE);
                }
                MenuSelectionManager.defaultManager().clearSelectedPath();
            }
            catch (RuntimeException ex) {
                this.realUngrabWindow();
                throw ex;
            }
            catch (Error error) {
                this.realUngrabWindow();
                throw error;
            }
        }
        
        @Override
        public void componentResized(ComponentEvent componentEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void componentMoved(ComponentEvent componentEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void componentShown(ComponentEvent componentEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void componentHidden(ComponentEvent componentEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void windowClosing(WindowEvent windowEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void windowClosed(WindowEvent windowEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void windowIconified(WindowEvent windowEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void windowDeactivated(WindowEvent windowEvent) {
            this.cancelPopupMenu();
        }
        
        @Override
        public void windowOpened(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowDeiconified(WindowEvent windowEvent) {
        }
        
        @Override
        public void windowActivated(WindowEvent windowEvent) {
        }
    }
    
    static class MenuKeyboardHelper implements ChangeListener, KeyListener
    {
        private Component lastFocused;
        private MenuElement[] lastPathSelected;
        private JPopupMenu lastPopup;
        private JRootPane invokerRootPane;
        private ActionMap menuActionMap;
        private InputMap menuInputMap;
        private boolean focusTraversalKeysEnabled;
        private boolean receivedKeyPressed;
        private FocusListener rootPaneFocusListener;
        
        MenuKeyboardHelper() {
            this.lastFocused = null;
            this.lastPathSelected = new MenuElement[0];
            this.menuActionMap = BasicPopupMenuUI.getActionMap();
            this.receivedKeyPressed = false;
            this.rootPaneFocusListener = new FocusAdapter() {
                @Override
                public void focusGained(FocusEvent focusEvent) {
                    Component oppositeComponent = focusEvent.getOppositeComponent();
                    if (oppositeComponent != null) {
                        BasicPopupMenuUI.MenuKeyboardHelper.this.lastFocused = oppositeComponent;
                    }
                    focusEvent.getComponent().removeFocusListener(this);
                }
            };
        }
        
        void removeItems() {
            if (this.lastFocused != null) {
                if (!this.lastFocused.requestFocusInWindow()) {
                    Window focusedWindow = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusedWindow();
                    if (focusedWindow != null && "###focusableSwingPopup###".equals(focusedWindow.getName())) {
                        this.lastFocused.requestFocus();
                    }
                }
                this.lastFocused = null;
            }
            if (this.invokerRootPane != null) {
                this.invokerRootPane.removeKeyListener(this);
                this.invokerRootPane.setFocusTraversalKeysEnabled(this.focusTraversalKeysEnabled);
                this.removeUIInputMap(this.invokerRootPane, this.menuInputMap);
                this.removeUIActionMap(this.invokerRootPane, this.menuActionMap);
                this.invokerRootPane = null;
            }
            this.receivedKeyPressed = false;
        }
        
        JPopupMenu getActivePopup(MenuElement[] array) {
            int i = array.length - 1;
            while (i >= 0) {
                MenuElement menuElement = array[i];
                if ((menuElement instanceof JPopupMenu)) {
                    return (JPopupMenu)menuElement;
                }
                --i;
            }
            return null;
        }
        
        void addUIInputMap(JComponent component, InputMap inputMap) {
            InputMap inputMap2 = null;
            InputMap parent;
            for (parent = component.getInputMap(2); parent != null && !(parent instanceof UIResource); parent = parent.getParent()) {
                inputMap2 = parent;
            }
            if (inputMap2 == null) {
                component.setInputMap(2, inputMap);
            }
            else {
                inputMap2.setParent(inputMap);
            }
            inputMap.setParent(parent);
        }
        
        void addUIActionMap(JComponent component, ActionMap actionMap) {
            ActionMap actionMap2 = null;
            ActionMap parent;
            for (parent = component.getActionMap(); parent != null && !(parent instanceof UIResource); parent = parent.getParent()) {
                actionMap2 = parent;
            }
            if (actionMap2 == null) {
                component.setActionMap(actionMap);
            }
            else {
                actionMap2.setParent(actionMap);
            }
            actionMap.setParent(parent);
        }
        
        void removeUIInputMap(JComponent component, InputMap inputMap) {
            InputMap inputMap2 = null;
            InputMap inputMap3 = component.getInputMap(2);
            while (inputMap3 != null) {
                if (inputMap3 == inputMap) {
                    if (inputMap2 == null) {
                        component.setInputMap(2, inputMap.getParent());
                        break;
                    }
                    inputMap2.setParent(inputMap.getParent());
                    break;
                }
                else {
                    inputMap2 = inputMap3;
                    inputMap3 = inputMap3.getParent();
                }
            }
        }
        
        void removeUIActionMap(JComponent component, ActionMap actionMap) {
            ActionMap actionMap2 = null;
            ActionMap actionMap3 = component.getActionMap();
            while (actionMap3 != null) {
                if (actionMap3 == actionMap) {
                    if (actionMap2 == null) {
                        component.setActionMap(actionMap.getParent());
                        break;
                    }
                    actionMap2.setParent(actionMap.getParent());
                    break;
                }
                else {
                    actionMap2 = actionMap3;
                    actionMap3 = actionMap3.getParent();
                }
            }
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            if (!(UIManager.getLookAndFeel() instanceof BasicLookAndFeel)) {
                this.uninstall();
                return;
            }
            MenuElement[] selectedPath = ((MenuSelectionManager)changeEvent.getSource()).getSelectedPath();
            JPopupMenu lastPopup = this.getActivePopup(selectedPath);
            if (lastPopup != null && !lastPopup.isFocusable()) {
                return;
            }
            if (this.lastPathSelected.length != 0 && selectedPath.length != 0 && !BasicPopupMenuUI.access$300(selectedPath[0], this.lastPathSelected[0])) {
                this.removeItems();
                this.lastPathSelected = new MenuElement[0];
            }
            if (this.lastPathSelected.length == 0 && selectedPath.length > 0) {
                JComponent c;
                if (lastPopup == null) {
                    if (selectedPath.length == 2 && selectedPath[0] instanceof JMenuBar && selectedPath[1] instanceof JMenu) {
                        c = (JComponent)selectedPath[1];
                        lastPopup = ((JMenu)c).getPopupMenu();
                    }
                    else {
                        return;
                    }
                }
                else {
                    Component component = lastPopup.getInvoker();
                    if ((component instanceof JFrame)) {
                        c = ((JFrame)component).getRootPane();
                    }
                    else if ((component instanceof JDialog)) {
                        c = ((JDialog)component).getRootPane();
                    }
                    else if ((component instanceof JApplet)) {
                        c = ((JApplet)component).getRootPane();
                    }
                    else {
                        while (!(component instanceof JComponent)) {
                            if (component == null) {
                                return;
                            }
                            component = component.getParent();
                        }
                        c = (JComponent)component;
                    }
                }
                this.lastFocused = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                this.invokerRootPane = SwingUtilities.getRootPane(c);
                if (this.invokerRootPane != null) {
                    this.invokerRootPane.addFocusListener(this.rootPaneFocusListener);
                    this.invokerRootPane.requestFocus(true);
                    this.invokerRootPane.addKeyListener(this);
                    this.focusTraversalKeysEnabled = this.invokerRootPane.getFocusTraversalKeysEnabled();
                    this.invokerRootPane.setFocusTraversalKeysEnabled(false);
                    this.menuInputMap = BasicPopupMenuUI.getInputMap(lastPopup, this.invokerRootPane);
                    this.addUIInputMap(this.invokerRootPane, this.menuInputMap);
                    this.addUIActionMap(this.invokerRootPane, this.menuActionMap);
                }
            }
            else if (this.lastPathSelected.length != 0 && selectedPath.length == 0) {
                this.removeItems();
            }
            else if (lastPopup != this.lastPopup) {
                this.receivedKeyPressed = false;
            }
            this.lastPathSelected = selectedPath;
            this.lastPopup = lastPopup;
            return;
        }
        
        @Override
        public void keyPressed(KeyEvent e) {
            this.receivedKeyPressed = true;
            MenuSelectionManager.defaultManager().processKeyEvent(e);
        }
        
        @Override
        public void keyReleased(KeyEvent e) {
            if (this.receivedKeyPressed) {
                this.receivedKeyPressed = false;
                MenuSelectionManager.defaultManager().processKeyEvent(e);
            }
        }
        
        @Override
        public void keyTyped(KeyEvent e) {
            if (this.receivedKeyPressed) {
                MenuSelectionManager.defaultManager().processKeyEvent(e);
            }
        }
        
        void uninstall() {
            synchronized (BasicPopupMenuUI.MENU_KEYBOARD_HELPER_KEY) {
                MenuSelectionManager.defaultManager().removeChangeListener(this);
                AppContext.getAppContext().remove(BasicPopupMenuUI.MENU_KEYBOARD_HELPER_KEY);
            }
        }
    }
}
